import { ILifetime } from "../Lifetime";
import { IServiceProvider } from "../ServiceProvider";
import { MockSetup } from "./ServiceCollection";
export declare type Key<E> = keyof E & (string | symbol);
export declare type MatchingProperties<T, E> = {
    [K in keyof E]: E[K] extends T ? K : never;
}[keyof E];
export declare type SelectorOptions<T, E> = {
    [key in MatchingProperties<T, E>]: key & Key<E>;
};
export declare type Selector<T, E> = Key<E> | ((e: SelectorOptions<T, E>) => Key<E>);
export declare type Factory<T, E> = (data: E) => T;
export declare type DependencyConstructor<T, E> = {
    new (props: E): T;
} | {
    new (): T;
};
export declare type StatefulDependencyConstructor<T, E, P> = {
    new (provider: E, props: P): T;
} | DependencyConstructor<T, E>;
export declare type DependencyOptions<T, E> = {
    factory: Factory<T, E>;
} | DependencyConstructor<T, E>;
export declare type LifetimeConstructor<T, E> = new (name: Key<E>, factory: Factory<T, E>) => ILifetime<T, E>;
export interface IServiceCollection<E> {
    replaceSingleton<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    replaceTransient<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    replaceScoped<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    replace<T>(Lifetime: LifetimeConstructor<T, E>, dependency: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    tryAddSingleton<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    tryAddTransient<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    tryAddScoped<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    tryAdd<T>(Lifetime: LifetimeConstructor<T, E>, dependency: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    addStateful<T, P>(constructor: StatefulDependencyConstructor<T, E, P>, selector: Selector<Factory<T, P>, E>): void;
    addSingleton<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    addTransient<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    addScoped<T>(options: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    add<T>(Lifetime: LifetimeConstructor<T, E>, dependency: DependencyOptions<T, E>, selector: Selector<T, E>): void;
    get<T>(item: Selector<T, E>): ILifetime<T, E> | undefined;
    remove<T>(item: Selector<T, E>): ILifetime<T, E> | undefined;
    build(): IServiceProvider<E>;
    buildMock(mock?: MockSetup<E>): IServiceProvider<E>;
}
