"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ScopedContext = exports.DependencyTracker = void 0;
var Errors_1 = require("../Errors");
var ServiceCollection_1 = require("../ServiceCollection");
var DependencyTracker = /** @class */ (function () {
    function DependencyTracker() {
        Object.defineProperty(this, "lookup", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: {}
        });
        Object.defineProperty(this, "ordered", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
    }
    Object.defineProperty(DependencyTracker.prototype, "depth", {
        get: function () {
            return this.ordered.length;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(DependencyTracker.prototype, "enter", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (key) {
            if (key in this.lookup)
                throw new Errors_1.CircularDependencyError(key, this.ordered);
            this.ordered.push(key);
            this.lookup[key] = true;
        }
    });
    Object.defineProperty(DependencyTracker.prototype, "leave", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (key) {
            this.ordered.pop();
            delete this.lookup[key];
        }
    });
    return DependencyTracker;
}());
exports.DependencyTracker = DependencyTracker;
var ScopedContext = /** @class */ (function () {
    function ScopedContext(lifetimes) {
        Object.defineProperty(this, "lifetimes", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "proxy", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "dependencyTracker", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new DependencyTracker()
        });
        Object.defineProperty(this, "scope", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: {}
        });
        var self = this;
        this.lifetimes = lifetimes;
        this.proxy = new Proxy({}, {
            get: function (target, prop) { return self.provide(prop); },
        });
    }
    Object.defineProperty(ScopedContext.prototype, "provide", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (selector) {
            var key = ServiceCollection_1.ServiceCollection.extractSelector(selector);
            var lifetime = this.lifetimes[key];
            if (!lifetime)
                throw new Errors_1.ExistenceDependencyError(key);
            this.dependencyTracker.enter(lifetime.name);
            var result = lifetime.provide(this);
            this.dependencyTracker.leave(lifetime.name);
            return result;
        }
    });
    return ScopedContext;
}());
exports.ScopedContext = ScopedContext;
